import 'package:flutter/material.dart';

enum AnimationType { normal, fadeIn }

///Main class,
///[context] es el BuildContext de la aplicacón en ese momento
///[child] es el widget a navegar
///[animation] Es el tipo de animación a utilizar
///[duration] Es la duración de la transicion
///[replacement] Indica si se reemplazará la pagina padre, para evitar volver a atras.
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions(
      {@required this.context,
      @required this.child,
      this.replacement = false,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300)}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  /// Push normal de la pagina
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push Replacement de la pagina
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Controlador para transición normal entre paginas
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controlador de la transición con fadeIn
  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
              child: child,
              opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.easeOut)));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }
}
